@@ -2,72 +2,46 @@ package Mojo::UserAgent::Role::Queued;
 use Mojo::Base '-role';
 use Scalar::Util 'weaken';
 
-our $VERSION = "0.03";
+our $VERSION = "0.04";
 
 has max_active => sub { shift->max_connections };
-has active         => 0;
 
 around start => sub {
   my ($orig, $self, $tx, $cb) = @_;
   if ($cb) {
-    $self->_start_queue($orig, {tx => $tx, cb => $cb});
+    $self->_enqueue($orig, {tx => $tx, cb => $cb});
   }
   else {
     return $orig->($self, $tx); # Blocking calls skip the queue
   }
 };
 
-sub _start_queue {
+sub _enqueue {
   my ($self, $original_start, $job) = @_;
   $self->{'jobs'} ||= [];
   push @{$self->{'jobs'}}, $job;
-  unless ($self->{'timer'}) {
-      my $this = $self;
-      weaken $this;
-      my $orig = $original_start;
-      weaken $orig;
-      $self->{'timer'} = Mojo::IOLoop->recurring(0 => sub { $this->_process($orig); });
-  }
-}
-
-sub _stop_queue {
-  my ($self) = @_;
-  if ($self->{'timer'}) {
-    Mojo::IOLoop->remove($self->{'timer'});
-    $self->{'timer'} = undef;
-  }
-  return $self;
+  $self->_process($original_start);
 }
 
 sub _process {
   my ($self, $original_start) = @_;
   state $start //= $original_start;
-
+  state $active //= 0;
   # we have jobs and can run them:
-  while ($self->active < $self->max_active
+  while ($active < $self->max_active
     and my $job = shift @{$self->{'jobs'}})
   {
     my ($tx, $cb) = ($job->{tx}, $job->{cb});
-    $self->active($self->active + 1);
-    $start->( $self, $tx,
-              sub {
-                    my ($ua, $tx1) = @_;
-                    $ua->active($ua->active - 1);
-                    $cb->($ua, $tx1);
-                    $ua->_process();
-        });
+    $active++;
+    weaken $self;
+    $tx->on(finish => sub { $active--; $self->_process() });
+    $start->( $self, $tx, $cb );
   }
-  if (scalar @{$self->{'jobs'}} == 0 && $self->active == 0) {
+  if (scalar @{$self->{'jobs'}} == 0 && $active == 0) {
     $self->emit('stop_queue');
-    $self->_stop_queue();    # the timer shouldn't run STAM.
   }
 }
 
-before DESTROY => sub {
-  my ($self) = shift;
-  $self->_stop_queue();
-};
-
 
 1;
 __END__
@@ -93,7 +67,13 @@ Mojo::UserAgent::Role::Queued - A role to process non-blocking requests in a rat
                   $tx->res->dom->at('title')->text;
             }
            });
-   }
+   };
+   # works with promises, too:
+  my @p = map {
+    $ua->get_p($_)->then(sub { pop->res->dom->at('title')->text })
+      ->catch(sub { say "Error: ", @_ })
+  } @big_list_of_urls;
+   Mojo::Promise->all(@p)->wait;
  
 
 =head1 DESCRIPTION
@@ -125,12 +105,6 @@ Emitted when the queue has been emptied of all pending jobs.
 
 L<Mojo::UserAgent::Role::Queued> has the following attributes:
 
-=head2 active
-
-    print "Number of jobs running is: ", $ua->active;
-
-Current number of active non-blocking transactions being processed.
-
 =head2 max_active
 
     $ua->max_active(5);  # execute no more than 5 transactions at a time.
